import React, { createContext, useContext } from 'react';
import Injector, { Provide, ProvideValue, Provider } from '../core/injector';

type GetService = (provide: Provide) => ProvideValue;

type ProviderProps = {
  providers: Provider[];
  children: React.ReactNode;
};

type ConsumerProps = {
  provides: Provide[];
  children?: (
    services: ProvideValue[],
    getService: GetService
  ) => React.ReactNode;
};

const Context = createContext<Injector>(new Injector());

const Provider = (props: ProviderProps): React.ReactElement => {
  const parentInjector = useContext(Context);
  const { providers } = props;
  const injector = new Injector(providers, parentInjector);
  return <Context.Provider value={injector}>{props.children}</Context.Provider>;
};

const Consumer = (props: ConsumerProps): React.ReactNode => {
  const parentInjector = useContext(Context);
  const getService: GetService = (provide) => parentInjector.get(provide);
  const { provides = [] } = props;
  const services = provides.map((provide) => getService(provide));
  return typeof props.children === 'function'
    ? props.children(services, getService)
    : props.children;
};

export { Context, Provider, Consumer };
